/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

// This parser will send sysex to a K2500 to cause its lights to flash.

#include "K2500BlinkenLightsParser.h"
#include "data_structs.h"
#include "ParserRegistration.h"
#include "sched.h"

static Bool running;
static ioref ou_node;
static int pos;

static ScheduleTime func(void*)
{
	if (!running)
		return NEVER;
	unsigned char button[] = { 0x40, 0x41, 0x42, 0x47, 0x44, 0x43, 0x46, 0x45 };
	unsigned char message[] = { 0xf0, // start of sysex
								0x07, // Kurzweil Manufacturer ID
								0x00, // Device ID
								0x78, // Product Identifier
								0x14, // PANEL message type
								0x09, // button down
								button[pos], // button identifier
								0x00, // unused
								0x08, // button up
								button[pos], // button identifier
								0x00, // unused
								0xf7}; // end of sysex
	MIDI_msg m;
	m.node = ou_node;
	m.set_sys_ex(message, 12);
	oap->write(&m);
	pos = (pos+1) % 8;
	return 2000;
}

K2500BlinkenLightsParser::
K2500BlinkenLightsParser()
{
	pos = 0;
	ou_node = oap->open_output(); // open the default output
	running = True;
	task_data td(func, NULL, oap->time()+2000);
	schedp->insert(td);
}

K2500BlinkenLightsParser::
~K2500BlinkenLightsParser()
{
	running = False;
}

static parser* newFunc()
{
	return (new K2500BlinkenLightsParser());
}

static ParserRegistration reg(newFunc, "K2500BlinkenLightsParser");

// Author: Patrick Pelletier
