/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

// This parser is an example of how to use the Rogus score player. The MIDI file
// "test_MIDI_file" from the current directory is read in, and each key on the
// input device is mapped to play this score at a different speed and transposition.

#include "NotePairScoreParser.h"
#include "data_structs.h"
#include "Transposer.h"
#include "NotePairScorePlayer.h"
#include <math.h>
#include "ParserRegistration.h"
#include "Filedata.h"

static NotePairScorePlayer* sp_array[128];

static
double
log2(double x)
{
	return (log(x)/log(2));
}

static
void
my_output_function(const NotePairScoreElement& npse, NotePairScoreScale scale, ScheduleTime now)
{
	// extract transposition information from scale
	int t = -int(log2(scale)*12 + 0.5);
	// (actually, there shouldn't be any note-offs, since they're all paired up)
	if (npse.msg.msgt() == MIDI_msg::note_on || npse.msg.msgt() == MIDI_msg::note_off)
	{
		NotePairScoreElement mynpse = npse;
		int key = mynpse.msg.key();
		key += t;
		// do octave wrapping
		while (key < 0)
			key += 12;
		while (key > 127)
			key -= 12;
		mynpse.msg.set_key(key);
		mynpse.play(scale, now);
	}
	else
		npse.play(scale, now);
}

static
void
note_on( MIDI_msg *m ) 
{
	if (m->d2 == 0)
		sp_array[m->d1]->stop();
	else
		sp_array[m->d1]->start();
}

static
void
note_off( MIDI_msg *m )
{
	sp_array[m->d1]->stop();
}

NotePairScoreParser::
NotePairScoreParser( NotePairScorePtr myNotePairScore )
{
	oap->open_input(); // open the default input
	
	for (int i = 0; i < 128; i++)
	{
		sp_array[i] = new NotePairScorePlayer(myNotePairScore, my_output_function);
//		sp_array[i]->addFilter(new Transposer(i-60));
		sp_array[i]->setScale((NotePairScoreScale)(pow(2, (((float)(60-i))/12))));
		sp_array[i]->setLoop(True);
	}

	msg_hnd[MIDI_msg::note_on] = note_on;
	msg_hnd[MIDI_msg::note_off] = note_off;
}

NotePairScoreParser::
~NotePairScoreParser()
{
	int i;
	for (i = 0; i < 128; i++)
	{
		sp_array[i]->stop();
		delete sp_array[i];
	}
}

static void warnfunc(const char* s)
{
	ew.w("%s",s);
}

static parser* newFunc()
{
	FiledataPtr fdp = new Filedata("test_MIDI_file",warnfunc);
	ScoreCreationInfo sci(fdp);
	sci.addRemainingTracks(oap->open_output());
	return (new NotePairScoreParser(new NotePairScore(&sci)));
}

static ParserRegistration reg(newFunc, "NotePairScoreParser");

// Authors: Ben Denckla, Patrick Pelletier
