/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

/*
This file gives example of how to create a MIDI_parser with hooks into your own MIDI input 
processing functions.  In this case, the processing is a MIDI scope.
*/

#include "print_parser.h"
#include "ParserRegistration.h"

static void note_off(  MIDI_msg *m )
{
	tw.pr("chan %X note %3d off vel %3d\n", m->chan(), m->d1, m->d2 );
}

static void note_on(  MIDI_msg *m )
{
	tw.pr("time: %8lX chan %X note %3d on vel %3d\n", m->time, m->chan(), m->d1, m->d2 );
}

static void poly_press(  MIDI_msg *m )
{
	tw.pr("chan %X note %3d press   %3d\n", m->chan(), m->d1, m->d2 );
}

static void program_chn(  MIDI_msg *m )
{
	tw.pr("chan %X prog %3d\n", m->chan(), m->d1 );
}

static void chan_press(  MIDI_msg *m )
{
	tw.pr("chan %X press %3d\n", m->chan(), m->d1);
}

static void pitch_bend(  MIDI_msg *m ) 
{
	tw.pr("chan %X bend %d\n", m->chan(), m->bend() );
}

// for CONTROL_CHANGE()
static void modulation(  MIDI_msg *m )
{tw.pr( "MODULATION AT %d.", m->d2);}  

static void master_volume(  MIDI_msg *m )
{tw.pr(" MASTER VOLUME AT %d.", m->d2);}

static void local_control_switch(  MIDI_msg *m )
{
   // what is the meaning of channel for this message?
	switch (m->d2)
	{
		case   0: tw.pr("LOCAL CONTROL OFF.\n"); break;
		case 127: tw.pr("LOCAL CONTROL ON.\n");  break;
		default:  tw.pr("LOCAL CONTROL ERROR: NEITHER 0 NOR 127.\n");
	}
}

static void all_notes_off(  MIDI_msg* )
{
	tw.pr( " ALL NOTES OFF.\n" );
}

static void channels_equals_voices(  MIDI_msg* )
{
	tw.pr(" MONO MODE: ON (NUMBER OF CHANNELS EQUALS NUMBER OF VOICES).\n" );
}

static void set_channels(  MIDI_msg *m )
{tw.pr(" MONO MODE: ON (NUMBER OF CHANNELS SET TO %d).\n", m->d2 );}

static void omni_poly_mono(  MIDI_msg *m )
{
	switch (m->d1)
	{
	case 124:                                          // Omni Mode Off (All Notes Off)
			tw.pr( " omni off.\n" ); break;
	case 125:                                          // Omni Mode On (All Notes Off)
			tw.pr( " omni on.\n" ); break;
	case 126:                                          // Mono Mode On (Poly Mode Off/All Notes Off)
		if (m->d2 == 0)
			channels_equals_voices(m);
		else
			set_channels(m);
		break;
	case 127:                                          // Poly Mode On (Mono Mode Off/All Notes Off)
		tw.pr(" POLY MODE: ON.\n" ); break;
	}
}

static void control_chn(  MIDI_msg *m)
{
	tw.pr( "chan %X ", m->chan() );
	switch (m->d1) {
	case 1:                                            // Continuous Controller 1 MSB (Modulation bender)
		modulation(m); break;
	case 7:                                            // Volume
		master_volume(m); break;
	case 122:                                     // CHANNEL MODE MESSAGES:
	   local_control_switch(m); break;
	case 123:                                          // All Notes Off
		if (m->d2 == 0)
			all_notes_off(m);
		break;
	case 124:                                          // Omni Mode Off (All Notes Off)
	case 125:                                          // Omni Mode On (All Notes Off)
	case 126:                                          // Mono Mode On (Poly Mode Off/All Notes Off)
	case 127:                                          // Poly Mode On (Mono Mode Off/All Notes Off)
		omni_poly_mono(m);
		break;
	default:
		tw.pr(" control change %d", m->d1);
	}
	tw.pr( "\n" );
}

static char* sys_ex_man_ids[] = {
	"", "Sequential", "IDP", "Octave-Plateau", "Moog", "Passport Designs", "Lexicon", "Kurzweil", // 00 - 07
	"Fender", "", "AKG Acoustics", "Voyce Music", "Waveframe Corp", "ADA Signal Processors", "Garfield Electronics", "Ensoniq", // 8 - F
	"Oberheim", "Apple Computer", "Grey Matter Response", "Mimetics", "Palm Tree Instruments", "JL Cooper", "Lowrey", "Adams-Smith", // 10 - 17
	"Emu Systems", "Harmony Systems", "ART", "Baldwin", "Eventide", "Inventronics", "Key Concepts", "Clarity", // 18 - 1F
	"Passac", "Siel", "Synthaxe", "Stepp", "Hohner", "Twister", "Solton", "Jellinghaus MS", // 20 - 27
	"Southworth", "PPG", "JEN", "SSL Limited", "Audio Veritrieb-P, Struven", "Neve", "Soundtracs Ltd.", "Elka", // 28 - 2F
	"Dynacord", "Intercontinental Electronics", "Drawmer", "Clavia Digital Instruments", "Zyklus Limited", "General Electro Music", "Cheetah Marketing", "C. T. M.", // 30 - 37
	"Simmons", "Soundcraft Electronics", "Steinberg Digital Audio", "Wersi", "", "", "", "", // 37 - 3F
	"Kawai", "Roland", "Korg", "Yamaha", "Casio", "Moridaira", "Kamiya Studio", "Akai", // 40 - 47
	"Japan Victor", "Mesosha", "Hoshino Gakki", "Fujitsu Electronics", "Sony", "Nisshin Onpa", "TEAC Corp.", "System Product", // 48 - 4F
	"Matushita Electric", "Fostex" // 50 - 51
};

static char* sys_ex_3byte_man_ids[] = {
	"", "", "", "", "", "", "", "Digital Music Corp.", // 00 - 07
	"IOTA Systems", "", "", "IVL Technologies", "Southern Music Systems", "Lake Butler Sound Co.", "", "", // 08 - 0F
	"DOD Electronics", "", "", "", "Fretworks", "KAT", "Opcode", "", // 10 - 17
	"Spatial Sound", "KMX", "", "", "360 Systems", "", "", "", // 18-1F
	"Axxes", "Orban" // 20 - 21
};

static const char* kind_of_sysex(int l, const unsigned char* d)
{
	static char buf[80];
	if (l < 3)
		return "unknown";
	unsigned char id = d[1];
	const char* manufacturer;
	if (id == 0)
	{
		if (l < 6)
			return "unknown";
		manufacturer = (d[3] < 0x22) ? sys_ex_3byte_man_ids[d[3]] : "";
	}
	else if (id < 0x7d)
	{
		manufacturer = (id < 0x52) ? sys_ex_man_ids[id] : "";
	}
	if (id < 0x7d)
	{
		if (strlen(manufacturer) == 0)
			manufacturer = "unknown";
		sprintf(buf, "manufacturer %s", manufacturer);
		return buf;
	}
	if (id == 0x7d)
		return "non-commercial";
	if (id == 0x7e)
	{
		if (l < 6)
			return "unknown";
		switch (d[3])
		{
		case 1:
			return "sample dump header";
		case 2:
			return "sample dump data packet";
		case 3:
			return "sample dump request";
		case 4:
			return "MIDI time code setup";
		case 5:
			return "sample dump extensions";
		case 6:
			return "inquiry message";
		case 0x7c:
			return "wait";
		case 0x7d:
			return "cancel";
		case 0x7e:
			return "NAK";
		case 0x7f:
			return "ACK";
		default:
			return "unknown";
		}
	}
	if (id == 0x7f)
		return "real time";
	ew.w("Sanity check failure");
	return "unknown";
}

static void sys_ex(  MIDI_msg *m )
{
	if (m->ed->st != smsgt::sys_ex)
	{
		ew.w("Sanity check failure");
		return;
	}
//	tw.pr("sysex ID %d\n", m->ed->sys_ex.d[1]);
	tw.pr("sysex %s:",kind_of_sysex(m->ed->sys_ex.l, m->ed->sys_ex.d));
	int i;
	for (i = 0; i < m->ed->sys_ex.l && i < 16; i++)
		tw.pr(" %02X", m->ed->sys_ex.d[i]);
	if (i < m->ed->sys_ex.l)
		tw.pr(" ...");
	tw.pr("\n");
}

static void system_msg(  MIDI_msg *m )
{
	switch (m->smsgt())
	{
	// System Exclusive message
	case smsgt::ext: sys_ex(m); break;
	
	// System Common messages
	case smsgt::mtc: 				 tw.pr("MTC.\n");							 break;
	case smsgt::song_position:       tw.pr("SONG POSITION: %d.\n", m->dat14() ); break;
	case smsgt::song_select:         tw.pr("SONG SELECT: SONG #%d.\n", m->d1);	 break;
//	case smsgt::sys_com_undefined_1: tw.pr("SYSTEM COMMON UNDEFINED 1.\n");      break;
	case smsgt::sys_com_undefined_2: tw.pr("SYSTEM COMMON UNDEFINED 2.\n");      break;
	case smsgt::tune_request:        tw.pr("TUNE REQUEST.\n");                   break;
	case smsgt::end_sys_ex:          tw.pr("END OF SYSTEM EXCLUSIVE.\n");        break;
	
	// System Realtime messages
	case smsgt::timing_clock:        tw.pr("TIMING CLOCK.\n");                   break;
	case smsgt::sys_rea_undefined_1: tw.pr("SYSTEM REALTIME UNDEFINED 0.\n");    break;
	case smsgt::start:               tw.pr("START.\n");                          break;
	case smsgt::cont:                tw.pr("CONTINUE.\n");                       break;
	case smsgt::stop:                tw.pr("STOP.\n");                           break;
	case smsgt::sys_rea_undefined_2: tw.pr("SYSTEM REALTIME UNDEFINED 1.\n");    break;
	case smsgt::active_sensing:      tw.pr("ACTIVE SENSING.\n");                 break;
	case smsgt::system_reset:        tw.pr("SYSTEM RESET.\n");                   break;
	default:
		tw.pr( "ERROR (non-fatal, but bad): unrecognized system message\n" );
	}
}

print_parser::
print_parser()
{
	msg_hnd[MIDI_msg::note_off]    = note_off;
	msg_hnd[MIDI_msg::note_on]     = note_on;
	msg_hnd[MIDI_msg::poly_press]  = poly_press;
	msg_hnd[MIDI_msg::control_chn] = control_chn;
	msg_hnd[MIDI_msg::program_chn] = program_chn;
	msg_hnd[MIDI_msg::chan_press]  = chan_press;
	msg_hnd[MIDI_msg::pitch_bend]  = pitch_bend;
	msg_hnd[MIDI_msg::system]  = system_msg;
	
	oap->open_input(); // open the default input
}

static parser* newFunc()
{
	return (new print_parser());
}

static ParserRegistration reg(newFunc, "print_parser");

// Authors: Ben Denckla, Herb Yang
